@@ -1283,7 +1283,7 @@ class SchemaView(object):
             raise ValueError(f'Unrecognized range: {r}')
         return range_types
 
-    def slot_range_as_union(self, slot: SlotDefinition) -> List[EnumDefinitionName]:
+    def slot_range_as_union(self, slot: SlotDefinition) -> List[ElementName]:
         """
         Returns all applicable ranges for a slot
 
@@ -1323,6 +1323,36 @@ class SchemaView(object):
 
         return list(dict.fromkeys(slots_list))
 
+    @lru_cache()
+    def get_slots_by_enum(self, enum_name: ENUM_NAME = None) -> List[SlotDefinition]:
+        """Get all slots that use a given enum: schema defined, attribute, or slot_usage.
+
+        :param enum_name: enum in consideration
+        :return: list of slots, either schem or both class attribute defined
+        """
+        enum_slots = [s for s in self.all_slots().values() if s.range == enum_name]
+        for class_definition in self.all_classes().values():
+            if class_definition.slot_usage:
+                for slot_definition in class_definition.slot_usage.values():
+                    if slot_definition.range == enum_name:
+                        enum_slots.append(slot_definition)
+        return enum_slots
+
+    def is_type_percent_encoded(self, type: TypeDefinitionName) -> bool:
+        """
+        True if type is has a percent_encoded annotation.
+
+        This is true for type fields that are the range of identifier columns,
+        where the identifier is not guaranteed to be a valid URI or CURIE
+
+        :param type:
+        :return:
+        """
+        id_slot_ranges = self.type_ancestors(type)
+        for t in id_slot_ranges:
+            anns = self.get_type(t).annotations
+            return "percent_encoded" in anns
+
     @lru_cache()
     def usage_index(self) -> Dict[ElementName, List[SchemaUsage]]:
         """
